//-----------------------------------------------------------------------------
#ifndef WizardSequencerControlH
#define WizardSequencerControlH WizardSequencerControlH
//-----------------------------------------------------------------------------
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>
#include <mvIMPACT_CPP/mvIMPACT_acquire_GenICam.h>
#include "PropViewFrame.h"
#include "ValuesFromUserDlg.h"


//-----------------------------------------------------------------------------
class WizardSequencerControl : public OkAndCancelDlg
//-----------------------------------------------------------------------------
{
public:
    explicit WizardSequencerControl( PropViewFrame* pParent, const wxString& title, mvIMPACT::acquire::Device* pDev, size_t displayCount, SequencerSetToDisplayMap& setToDisplayTable );
    virtual ~WizardSequencerControl();
    const SequencerSetToDisplayMap&     GetSetToDisplayTable( void );

protected:
    //-----------------------------------------------------------------------------
    /// \brief GUI elements for a single sequencer set
    struct SequencerSetControls
            //-----------------------------------------------------------------------------
    {
        wxPanel* pSetPanel;
        wxBoxSizer* pSetVerticalSizer;
        wxBoxSizer* pSetHorizontalSizer;
        wxBoxSizer* pOptionsSizer;
        wxPropertyGrid* pSetPropertyGrid;
        wxSpinCtrl* pSequencerSetNextSC;
        wxComboBox* pDisplayToUseCB;
    };
    typedef std::map<int64_type, SequencerSetControls*> SetNrToControlsMap;

private:
    //-----------------------------------------------------------------------------
    enum TWidgetIDs_SequencerControl
    //-----------------------------------------------------------------------------
    {
        widMainFrame = wxID_HIGHEST,
        widDisplayComboBox,
        widNextSetSpinControl,
        widSettingsCheckBox,
        widAddAnotherSetButton,
        widRemoveSelectedSetButton,
        widAutoAssignSetsToDisplays,
        widStaticBitmapWarning
    };

    //-----------------------------------------------------------------------------
    enum TSequencerWizardErrorTypes
    //-----------------------------------------------------------------------------
    {
        swetReferenceToInactiveSet,
        swetSetsNotBeingReferenced
    };

    //-----------------------------------------------------------------------------
    /// \brief Necessary Information for Sequencer Set Management Errors
    struct SequencerErrorInfo
            //-----------------------------------------------------------------------------
    {
        int64_type setNumber;
        TSequencerWizardErrorTypes errorType;
        int64_type nextSet;
    };

    PropViewFrame*                      pParent_;
    GenICam::SequencerControl           sc_;
    mutable SequencerSetToDisplayMap    setToDisplayTable_;
    std::vector<SequencerErrorInfo>     currentErrors_;
    std::map<HOBJ, bool>                sequencerFeaturesMap_;
    std::map<int64_type, int64_type>    sequencerNextSetMap_;
    SetNrToControlsMap                  sequencerSetControlsMap_;
    int64_type                          startingSequencerSet_;
    int64_type                          maxSequencerSetNumber_;
    size_t                              displayCount_;
    bool                                boCounterDurationCapability_;

    wxNotebook*                         pSetSettingsNotebook_;
    wxStaticBitmap*                     pStaticBitmapWarning_;
    wxStaticText*                       pInfoText_;

    static const wxString               s_displayStringPrefix_;
    static const wxString               s_sequencerSetStringPrefix_;

    void                                AnalyzeSequencerSetFeatures( void );
    void                                AddError( SequencerErrorInfo& errorInfo );
    void                                CheckIfActiveSetsAreReferencedAtLeastOnce( void );
    void                                CheckIfReferencedSetsAreActive( void );
    void                                ComposeSetPanel( size_t displayCount, int64_type& setCount, int64_type setNumber, bool boInsertOnFirstFreePosition );
    void                                HandleAutomaticNextSetSettingsOnInsert( unsigned int tabPageNumber, unsigned int setNumber );
    void                                HandleAutomaticNextSetSettingsOnRemove( unsigned int tabPageNumber, unsigned int setNumber );
    void                                InquireCurrentlyActiveSequencerSets( void );
    void                                UpdateError( const wxString& msg, const wxBitmap& icon ) const;
    void                                UpdateErrors( void ) const;
    void                                UpdatePropertyGrid( SequencerSetControls* pSelectedSequencerSet );
    void                                LoadSequencerSet( int64_type setNumber );
    void                                LoadSequencerSets( void );
    void                                RemoveError( SequencerErrorInfo& errorInfo );
    void                                RemoveSetPanel( void );
    void                                ReadPropertiesOfSequencerSet( SetNrToControlsMap::iterator& it );
    bool                                SaveSequencerSets( void );

    //----------------------------------GUI Functions----------------------------------
    void                                OnBtnApply( wxCommandEvent& e );
    void                                OnBtnCancel( wxCommandEvent& e );
    void                                OnBtnOk( wxCommandEvent& e );
    void                                OnBtnAddAnotherSet( wxCommandEvent& e );
    void                                OnBtnRemoveSelectedSet( wxCommandEvent& e );
    void                                OnBtnAutoAssignSetsToDisplays( wxCommandEvent& e );
    void                                OnNextSetSpinControlChanged( wxSpinEvent& e );
    void                                OnPropertyChanged( wxPropertyGridEvent& e );
    void                                OnSettingsCheckBoxChecked( wxCommandEvent& e );

    DECLARE_EVENT_TABLE()
};

#endif // WizardSequencerControlH
